@@ -20,6 +20,8 @@ import datetime
 import munch
 import six
 
+from openstack import resource
+
 _IMAGE_FIELDS = (
     'checksum',
     'container_format',
@@ -38,15 +40,19 @@ _SERVER_FIELDS = (
     'addresses',
     'adminPass',
     'created',
+    'description',
     'key_name',
     'metadata',
     'networks',
+    'personality',
     'private_v4',
     'public_v4',
     'public_v6',
+    'server_groups',
     'status',
     'updated',
     'user_id',
+    'tags',
 )
 
 _KEYPAIR_FIELDS = (
@@ -263,11 +269,18 @@ class Normalizer(object):
         return ret
 
     def _normalize_image(self, image):
-        new_image = munch.Munch(
-            location=self._get_current_location(project_id=image.get('owner')))
+        if isinstance(image, resource.Resource):
+            image = image.to_dict(ignore_none=True, original_names=True)
+            location = image.pop(
+                'location',
+                self._get_current_location(project_id=image.get('owner')))
+        else:
+            location = self._get_current_location(
+                project_id=image.get('owner'))
+            # This copy is to keep things from getting epically weird in tests
+            image = image.copy()
 
-        # This copy is to keep things from getting epically weird in tests
-        image = image.copy()
+        new_image = munch.Munch(location=location)
 
         # Discard noise
         self._remove_novaclient_artifacts(image)
@@ -315,7 +328,7 @@ class Normalizer(object):
         new_image['is_protected'] = protected
         new_image['locations'] = image.pop('locations', [])
 
-        metadata = image.pop('metadata', {})
+        metadata = image.pop('metadata', {}) or {}
         for key, val in metadata.items():
             properties.setdefault(key, val)
 
@@ -451,7 +464,8 @@ class Normalizer(object):
     def _normalize_server(self, server):
         ret = munch.Munch()
         # Copy incoming server because of shared dicts in unittests
-        server = server.copy()
+        # Wrap the copy in munch so that sub-dicts are properly munched
+        server = munch.Munch(server)
 
         self._remove_novaclient_artifacts(server)
 
@@ -460,20 +474,33 @@ class Normalizer(object):
 
         server['flavor'].pop('links', None)
         ret['flavor'] = server.pop('flavor')
+        # From original_names from sdk
+        server.pop('flavorRef', None)
 
         # OpenStack can return image as a string when you've booted
         # from volume
-        if str(server['image']) != server['image']:
-            server['image'].pop('links', None)
-        ret['image'] = server.pop('image')
+        image = server.pop('image', None)
+        if str(image) != image:
+            image = munch.Munch(id=image['id'])
+
+        ret['image'] = image
+        # From original_names from sdk
+        server.pop('imageRef', None)
+        # From original_names from sdk
+        ret['block_device_mapping'] = server.pop('block_device_mapping_v2', {})
 
         project_id = server.pop('tenant_id', '')
         project_id = server.pop('project_id', project_id)
 
         az = _pop_or_get(
             server, 'OS-EXT-AZ:availability_zone', None, self.strict_mode)
-        ret['location'] = self._get_current_location(
-            project_id=project_id, zone=az)
+        # the server resource has this already, but it's missing az info
+        # from the resource.
+        # TODO(mordred) create_server is still normalizing servers that aren't
+        # from the resource layer.
+        ret['location'] = server.pop(
+            'location', self._get_current_location(
+                project_id=project_id, zone=az))
 
         # Ensure volumes is always in the server dict, even if empty
         ret['volumes'] = _pop_or_get(
@@ -484,7 +511,7 @@ class Normalizer(object):
             'has_config_drive', server.pop('config_drive', False))
         ret['has_config_drive'] = _to_bool(config_drive)
 
-        host_id = server.pop('hostId', None)
+        host_id = server.pop('hostId', server.pop('host_id', None))
         ret['host_id'] = host_id
 
         ret['progress'] = _pop_int(server, 'progress')
@@ -497,7 +524,19 @@ class Normalizer(object):
                 'OS-EXT-STS:task_state',
                 'OS-EXT-STS:vm_state',
                 'OS-SRV-USG:launched_at',
-                'OS-SRV-USG:terminated_at'):
+                'OS-SRV-USG:terminated_at',
+                'OS-EXT-SRV-ATTR:hypervisor_hostname',
+                'OS-EXT-SRV-ATTR:instance_name',
+                'OS-EXT-SRV-ATTR:user_data',
+                'OS-EXT-SRV-ATTR:host',
+                'OS-EXT-SRV-ATTR:hostname',
+                'OS-EXT-SRV-ATTR:kernel_id',
+                'OS-EXT-SRV-ATTR:launch_index',
+                'OS-EXT-SRV-ATTR:ramdisk_id',
+                'OS-EXT-SRV-ATTR:reservation_id',
+                'OS-EXT-SRV-ATTR:root_device_name',
+                'OS-SCH-HNT:scheduler_hints',
+        ):
             short_key = key.split(':')[1]
             ret[short_key] = _pop_or_get(server, key, None, self.strict_mode)
 
@@ -1063,7 +1102,10 @@ class Normalizer(object):
 
     def _normalize_stack(self, stack):
         """Normalize Heat Stack"""
-        stack = stack.copy()
+        if isinstance(stack, resource.Resource):
+            stack = stack.to_dict(ignore_none=True, original_names=True)
+        else:
+            stack = stack.copy()
 
         # Discard noise
         self._remove_novaclient_artifacts(stack)
@@ -1073,7 +1115,10 @@ class Normalizer(object):
         stack.pop('status', None)
         stack.pop('identifier', None)
 
-        stack_status = stack.pop('stack_status')
+        stack_status = None
+
+        stack_status = stack.pop('stack_status', None) or \
+            stack.pop('status', None)
         (action, status) = stack_status.split('_', 1)
 
         ret = munch.Munch(
@@ -1102,13 +1147,13 @@ class Normalizer(object):
                 ('tempate_description', 'template_description'),
                 ('timeout_mins', 'timeout_mins'),
                 ('tags', 'tags')):
-            value = stack.pop(old_name, None)
+            value = stack.get(old_name, None)
             ret[new_name] = value
             if not self.strict_mode:
                 ret[old_name] = value
         ret['identifier'] = '{name}/{id}'.format(
             name=ret['name'], id=ret['id'])
-        ret['properties'] = stack
+        # ret['properties'] = stack
         return ret
 
     def _normalize_machines(self, machines):
@@ -1146,3 +1191,37 @@ class Normalizer(object):
             location=self._get_identity_location(),
             properties={},
         )
+
+    def _normalize_containers(self, containers):
+        """Normalize Swift Containers"""
+        ret = []
+        for container in containers:
+            ret.append(self._normalize_container(container))
+        return ret
+
+    def _normalize_container(self, container):
+        """Normalize Swift Container."""
+
+        return munch.Munch(
+            name=container.get('name'),
+            bytes=container.get('bytes'),
+            count=container.get('count'),
+        )
+
+    def _normalize_objects(self, objects):
+        """Normalize Swift Objects"""
+        ret = []
+        for object in objects:
+            ret.append(self._normalize_object(object))
+        return ret
+
+    def _normalize_object(self, object):
+        """Normalize Swift Object."""
+
+        return munch.Munch(
+            name=object.get('name'),
+            bytes=object.get('_bytes'),
+            content_type=object.get('content_type'),
+            hash=object.get('_hash'),
+            last_modified=object.get('_last_modified'),
+        )
