@@ -156,7 +156,6 @@ Additional information about the services can be found in the
 :ref:`service-proxies` documentation.
 """
 import warnings
-import weakref
 
 import keystoneauth1.exceptions
 import requestsexceptions
@@ -165,6 +164,20 @@ import six
 from openstack import _log
 from openstack._meta import connection as _meta
 from openstack.cloud import openstackcloud as _cloud
+from openstack.cloud import _baremetal
+from openstack.cloud import _block_storage
+from openstack.cloud import _compute
+from openstack.cloud import _clustering
+from openstack.cloud import _coe
+from openstack.cloud import _dns
+from openstack.cloud import _floating_ip
+from openstack.cloud import _identity
+from openstack.cloud import _image
+from openstack.cloud import _network
+from openstack.cloud import _network_common
+from openstack.cloud import _object_store
+from openstack.cloud import _orchestration
+from openstack.cloud import _security_group
 from openstack import config as _config
 from openstack.config import cloud_region
 from openstack import exceptions
@@ -212,7 +225,22 @@ def from_config(cloud=None, config=None, options=None, **kwargs):
 
 
 class Connection(six.with_metaclass(_meta.ConnectionMeta,
-                                    _cloud._OpenStackCloudMixin)):
+                                    _cloud._OpenStackCloudMixin,
+                                    _baremetal.BaremetalCloudMixin,
+                                    _block_storage.BlockStorageCloudMixin,
+                                    _compute.ComputeCloudMixin,
+                                    _clustering.ClusteringCloudMixin,
+                                    _coe.CoeCloudMixin,
+                                    _dns.DnsCloudMixin,
+                                    _floating_ip.FloatingIPCloudMixin,
+                                    _identity.IdentityCloudMixin,
+                                    _image.ImageCloudMixin,
+                                    _network.NetworkCloudMixin,
+                                    _network_common.NetworkCommonCloudMixin,
+                                    _object_store.ObjectStoreCloudMixin,
+                                    _orchestration.OrchestrationCloudMixin,
+                                    _security_group.SecurityGroupCloudMixin
+                                    )):
 
     def __init__(self, cloud=None, config=None, session=None,
                  app_name=None, app_version=None,
@@ -299,9 +327,49 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         self._proxies = {}
         self.use_direct_get = use_direct_get
         self.strict_mode = strict
-        # Call the _OpenStackCloudMixin constructor while we work on
+        # Call the _*CloudMixin constructors while we work on
         # integrating things better.
         _cloud._OpenStackCloudMixin.__init__(self)
+        _baremetal.BaremetalCloudMixin.__init__(self)
+        _block_storage.BlockStorageCloudMixin.__init__(self)
+        _clustering.ClusteringCloudMixin.__init__(self)
+        _coe.CoeCloudMixin.__init__(self)
+        _compute.ComputeCloudMixin.__init__(self)
+        _dns.DnsCloudMixin.__init__(self)
+        _floating_ip.FloatingIPCloudMixin.__init__(self)
+        _identity.IdentityCloudMixin.__init__(self)
+        _image.ImageCloudMixin.__init__(self)
+        _network_common.NetworkCommonCloudMixin.__init__(self)
+        _network.NetworkCloudMixin.__init__(self)
+        _object_store.ObjectStoreCloudMixin.__init__(self)
+        _orchestration.OrchestrationCloudMixin.__init__(self)
+        _security_group.SecurityGroupCloudMixin.__init__(self)
+
+        # Allow vendors to provide hooks. They will normally only receive a
+        # connection object and a responsible to register additional services
+        vendor_hook = kwargs.get('vendor_hook')
+        if not vendor_hook and 'vendor_hook' in self.config.config:
+            # Get the one from profile
+            vendor_hook = self.config.config.get('vendor_hook')
+        if vendor_hook:
+            try:
+                # NOTE(gtema): no class name in the hook, plain module:function
+                # Split string hook into module and function
+                try:
+                    (package_name, function) = vendor_hook.rsplit(':')
+
+                    if package_name and function:
+                        import pkg_resources
+                        ep = pkg_resources.EntryPoint(
+                            'vendor_hook', package_name, attrs=(function,))
+                        hook = ep.resolve()
+                        hook(self)
+                except ValueError:
+                    self.log.warning('Hook should be in the entrypoint '
+                                     'module:attribute format')
+            except (ImportError, TypeError) as e:
+                self.log.warning('Configured hook %s cannot be executed: %s',
+                                 vendor_hook, e)
 
     @property
     def session(self):
@@ -310,7 +378,7 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
             # Hide a reference to the connection on the session to help with
             # backwards compatibility for folks trying to just pass
             # conn.session to a Resource method's session argument.
-            self.session._sdk_connection = weakref.proxy(self)
+            self.session._sdk_connection = self
         return self._session
 
     def add_service(self, service):
@@ -334,11 +402,19 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         # we get an adapter.
         if isinstance(service, six.string_types):
             service = service_description.ServiceDescription(service)
-        service_proxy = service._make_proxy(self)
 
-        # Register the proxy class with every known alias
+        # Directly invoke descriptor of the ServiceDescription
+        def getter(self):
+            return service.__get__(self, service)
+
+        # Register the ServiceDescription class (as property)
+        # with every known alias for a "runtime descriptor"
         for attr_name in service.all_types:
-            setattr(self, attr_name.replace('-', '_'), service_proxy)
+            setattr(
+                self.__class__,
+                attr_name.replace('-', '_'),
+                property(fget=getter)
+            )
 
     def authorize(self):
         """Authorize this Connection
